/*                  PARTICUL.INC for POVRAY 3.1
                    GEMELLI David
                    Version 1.1         4 janvier 1999

     For more details (in a readable english I hope) see PARTICUL.DOC
*/
// Rayon_Particule: rayon des spheres (0.1)
// Objet_Particule: particule (par defaut une sphere)
// Rot_Random: rotation aleatoire de la particule (0)
// 0 => rotation inactive
// 1 => rotation active
// Forme: forme de la zone a remplir (2 Cube)
// 0 => Sphere: Rayon (10)
// 1 => Cylindre: Rayon (5), Hauteur (10)
// 2 => Cube: Taille (10)
// 3 => Cone: Rayon_Bas (0), Rayon_Haut (5), Hauteur (10)
// Nb_Couches: nombre de couches (5)
// PMax: nombre maxi de particules par couche (20)
// PVariation: variation du nombre de particules (de bas en haut) (0)
// 0 => variation inactive
// 1 => variation active:
//        definir une macro M_PVariation
//        parametres en entree : hauteur (0-1) , initial (PMax), retour
//        valeur en sortie: le nombre de paticules pour cette hauteur
//        ex:
//        #macro M_PVariation(hauteur, initial, retour)
//             #declare retour = initial + (pow(hauteur, 2) * 20);
//        #end
// Offset: variation aleatoire maximale de la hauteur (0)
// EVariation: variation de l'ecart (de bas en haut) (0)
// 0 => ecart constant
// 1 => ecart variable:
//        definir une macro M_EVariation
//        parametres en entree : hauteur (0-1) , initial (EMax), retour
//        valeur en sortie: la distance pour cette hauteur
//        ex:
//        #macro M_EVariation(hauteur, initial, retour)
//             #declare retour = initial + ((hauteur / 2) * 20);
//        #end
// EMax: ecart maxi entre le centre du volume et le remplissage (0)
// E2Variation: variation de l'ecart (de bas en haut) (0)
// 0 => ecart constant
// 1 => ecart variable:
//        definir une macro M_E2Variation
//        parametres en entree : hauteur (0-1) , initial (EMax), retour
//        valeur en sortie: la distance pour cette hauteur
//        ex:
//        #macro M_E2Variation(hauteur, initial, retour)
//             #declare retour = initial + ((hauteur / 2) * 20);
//        #end
// E2Max: ecart maxi entre le remplissage de base et la surface (0)
// Anti_Collision: creer des particules qui ne se touchent pas (0)
// 0 => inactif
// 1 => actif (long 'parsing')
// Resultat:
// un objet 'Particules': union de toutes les particules
// un objet 'Container': forme a remplir

// test des variables de parametrage
#ifndef (Rayon_Particule)
     #debug "Rayon_particule indefinie, valeur par defaut 0.1\n"
     #local Rayon_Particule = 0.1;
#else
     #if (Rayon_Particule <= 0)
          #debug "Rayon_particule non valide, valeur par defaut 0.1\n"
          #local Rayon_Particule = 0.1;
     #end
#end
#ifndef (Objet_Particule)
     #debug "Objet_Particule indefinie, objet par defaut\n"
     #local Objet_Particule = sphere {<0, 0, 0>, Rayon_Particule}
#end
#ifndef (Rot_Random)
     #debug "Rot_Random indefinie, valeur par defaut 0 (inactif)\n"
     #local Rot_Random = 0;
#end
#ifndef (Forme)
     #debug "Forme indefinie, valeur par defaut 2 (Cube)\n"
     #local Forme = 2;
#end
#ifndef (Nb_Couches)
     #debug "Nb_Couches indefinie, valeur par defaut 5\n"
     #local Nb_Couches = 5;
#else
     #if (Nb_Couches <= 0)
          #debug "Nb_Couches non valide, valeur par defaut 5\n"
          #local Nb_Couches = 5;
     #end
#end
#ifndef (PMax)
     #debug "PMax indefinie, valeur par defaut 20\n"
     #local PMax = 20;
#else
     #if (PMax <= 0)
          #debug "PMax non valide, valeur par defaut 20\n"
          #local PMax = 20;
     #end
#end
#ifndef (PVariation)
     #debug "PVariation indefinie, valeur par defaut 0 (inactif)\n"
     #local PVariation = 0;
#end
#ifndef (M_PVariation)
     #if (PVariation = 1)
          #debug "M_PVariation indefinie, macro par defaut\n"
     #end
     #macro M_PVariation(hauteur, initial, retour)
          #declare retour = initial;
     #end
#end
#ifndef (Offset)
     #debug "Offset indefinie, valeur par defaut 0\n"
     #local Offset = 0;
#end
#ifndef (EVariation)
     #debug "EVariation indefinie, valeur par defaut 0 (inactif)\n"
     #local EVariation = 0;
#end
#ifndef (EMax)
     #if (EVariation = 1)
          #debug "EMax indefinie, valeur par defaut 0\n"
     #end
     #local EMax = 0;
#end
#ifndef (M_EVariation)
     #if (EVariation = 1)
          #debug "M_EVariation indefinie, macro par defaut\n"
     #end
     #macro M_EVariation(hauteur, initial, retour)
          #declare retour = initial;
     #end
#end
#ifndef (E2Variation)
     #debug "E2Variation indefinie, valeur par defaut 0 (inactif)\n"
     #local E2Variation = 0;
#end
#ifndef (E2Max)
     #if (E2Variation = 1)
          #debug "E2Max indefinie, valeur par defaut 0\n"
     #end
     #local E2Max = 0;
#end
#ifndef (M_E2Variation)
     #if (E2Variation = 1)
          #debug "Macro M_E2Variation indefinie, macro par defaut\n"
     #end
     #macro M_E2Variation(hauteur, initial, retour)
          #declare retour = initial;
     #end
#end
#ifndef (Anti_Collision)
     #debug "Anti_Collision indefinie, valeur par defaut 0 (pas de test)\n"
     #local Anti_Collision = 0;
#end

#local Tab = array[3][Nb_Couches][PMax]
#declare Forme_Sphere = 0;
#declare Forme_Cylindre = 1;
#declare Forme_Cube = 2;
#declare Forme_Cone = 3;
#local Seed0 = seed(21564);
#local Seed1 = seed(79523);
#local Seed2 = seed(85760);

// generation des particules
#local IndC = 0;
#declare retour = 0;
#while (IndC < Nb_Couches)
     #local IndC2 = IndC + 0.5;
     #debug "Couche:"
     #debug str(IndC + 1, 3, 0)
     #debug "/"
     #debug str(Nb_Couches, 3, 0)
     #local valid = 0;
     #local start = 1;
     #local Ind = 0;
     #local Ind2 = 0;
     M_PVariation(IndC / Nb_Couches, PMax, retour)
     #local retourP = retour;
     #debug "    Nb particules:"
     #debug str(retourP, 5, 0)
     #debug "\n"
     #while (Ind < retourP)
          M_EVariation(IndC / Nb_Couches, EMax, retour)
          #local ecarttmp = retour;
          M_E2Variation(IndC / Nb_Couches, E2Max, retour)
          #local ecart2tmp = retour;
          #if (valid = 0)
               #switch (Forme)
                    #case(Forme_Sphere)
                         #ifndef (Rayon)
                              #debug "Rayon de la zone indefini, valeur par defaut 10\n"
                              #local Rayon = 10;
                         #end
                         #local alpha1 = radians(360 * rand(Seed0));
                         #local rayontmp = Rayon * sin (radians ((IndC2 / Nb_Couches) * 180));
                         #local rayontmp = ecarttmp + ((ecart2tmp + rayontmp - ecarttmp) * rand(Seed1));
                         #local InterC =  Rayon * 2 / Nb_Couches;
                         #local Tab[0][IndC][Ind] = rayontmp * cos(alpha1);
                         #local Tab[1][IndC][Ind] = (-Rayon) + (IndC2 * InterC) + (Offset * rand(Seed2)) - (Offset / 2);
                         #local Tab[2][IndC][Ind] = rayontmp * sin(alpha1);
                         #declare Container =
                         sphere {
                              <0, 0, 0>, Rayon
                         }
                    #break
                    #case(Forme_Cylindre)
                         #ifndef (Rayon)
                              #debug "Rayon de la zone indefini, valeur par defaut 5\n"
                              #local Rayon = 5;
                         #end
                         #ifndef (Hauteur)
                              #debug "Hauteur de la zone indefinie, valeur par defaut 10\n"
                              #local Hauteur = 10;
                         #end
                         #local alpha1 = radians(360 * rand(Seed0));
                         #local rayontmp = ecarttmp + ((ecart2tmp + Rayon - ecarttmp) * rand(Seed1));
                         #local InterC =  Hauteur / Nb_Couches;
                         #local Tab[0][IndC][Ind] = rayontmp * cos(alpha1);
                         #local Tab[1][IndC][Ind] = (-Hauteur / 2) + (IndC2 * InterC) + (Offset * rand(Seed2)) - (Offset / 2);
                         #local Tab[2][IndC][Ind] = rayontmp * sin(alpha1);
                         #declare Container =
                         cylinder {
                              <0, -Hauteur / 2, 0>,
                              <0, Hauteur / 2, 0>, Rayon
                         }
                    #break
                    #case(Forme_Cube)
                         #ifndef (Taille)
                              #debug "Taille de la zone indefinie, valeur par defaut 10\n"
                              #local Taille = 10;
                         #end
                         #local InterC = Taille / Nb_Couches;
                         #local Tab[0][IndC][Ind] = ecarttmp + (((ecart2tmp + Taille - ecarttmp) * rand(Seed0)) - (Taille / 2));
                         #local Tab[1][IndC][Ind] = (-Taille / 2) + (IndC2 * InterC) + (Offset * rand(Seed1)) - (Offset / 2);
                         #local Tab[2][IndC][Ind] = ecarttmp + (((ecart2tmp + Taille - ecarttmp) * rand(Seed2)) - (Taille / 2));
                         #declare Container =
                         box {
                              <-Taille / 2, -Taille / 2, -Taille / 2>,
                              <Taille / 2, Taille / 2, Taille / 2>
                         }
                    #break
                    #case(Forme_Cone)
                         #ifndef (Rayon_Bas)
                              #debug "Rayon bas de la zone indefini, valeur par defaut 0\n"
                              #local Rayon_Bas = 0;
                         #end
                         #ifndef (Rayon_Haut)
                              #debug "Rayon haut de la zone indefini, valeur par defaut 5\n"
                              #local Rayon_Haut = 5;
                         #end
                         #ifndef (Hauteur)
                              #debug "Hauteur de la zone indefinie, valeur par defaut 10\n"
                              #local Hauteur = 10;
                         #end
                         #local InterR = (Rayon_Haut - Rayon_Bas) / Nb_Couches;
                         #local InterC = Hauteur / Nb_Couches;
                         #local rayontmp = Rayon_Bas + (InterR * IndC2);
                         #local alpha1 = radians(360 * rand(Seed0));
                         #local rayontmp = ecarttmp + ((ecart2tmp + rayontmp - ecarttmp) * rand(Seed1));
                         #local Tab[0][IndC][Ind] = rayontmp * cos(alpha1);
                         #local Tab[1][IndC][Ind] = (-Hauteur / 2) + (IndC2 * InterC) + (Offset * rand(Seed2)) - (Offset / 2);
                         #local Tab[2][IndC][Ind] = rayontmp * sin(alpha1);
                         #declare Container =
                         cone {
                              <0, -Hauteur / 2, 0>, Rayon_Bas,
                              <0, Hauteur / 2, 0>, Rayon_Haut
                         }
                    #break
               #end
          #end
          #if (Anti_Collision = 0)
               #local Ind2 = Ind;
          #else
               #if (start = 0)
                    #local rpx = Tab[0][IndC][Ind];
                    #local rpy = Tab[1][IndC][Ind];
                    #local rpz = Tab[2][IndC][Ind];
                    #local rpx2 = Tab[0][IndC][Ind2];
                    #local rpy2 = Tab[1][IndC][Ind2];
                    #local rpz2 = Tab[2][IndC][Ind2];
                    #local dist = sqrt(pow(rpx2-rpx,2)+pow(rpy2-rpy,2)+pow(rpz2-rpz,2));
                    #if (dist > Rayon_Particule * 2)
                         #local valid = 1;
                    #end
                    #if (valid = 1)
                         #local Ind2 = Ind2 + 1;
                    #else
                         #local Ind2 = 0;
                    #end
               #else
                    #local start = 0;
               #end
          #end
          #if (Ind2 = Ind)
               #local Ind = Ind + 1;
               #local Ind2 = 0;
               #local valid = 0;
          #end
     #end
     #local IndC = IndC + 1;
     M_PVariation(IndC / Nb_Couches, PMax, retour)
     #local retourP = retour;
#end

// L'objet Particules
#declare Particules =
union {
     #local IndC = 0;
     #while (IndC < Nb_Couches)
          #local Ind = 0;
          M_PVariation(IndC / Nb_Couches, PMax, retour)
          #while (Ind < retour)
               object {
                    Objet_Particule
                    #if (Rot_Random = 1)
                         rotate <360*rand(Seed0), 360*rand(Seed1), 360*rand(Seed2)>
                    #end
                    translate <Tab[0][IndC][Ind], Tab[1][IndC][Ind], Tab[2][IndC][Ind]>
               }
               #local Ind = Ind + 1;
          #end
          #local IndC = IndC + 1;
          M_PVariation(IndC / Nb_Couches, PMax, retour)
     #end
}
